C     *****************************************************************
C     MAIN CODE FOR T-COMP_Create
C     ******************************************************************
c  Program  T-COMP_Create.f   ---   Creates comparable radial models and batch file to run all.
c
c  		Read MODFLOW files.
c  		Extracts regional gridding and CONDUCTANCES 
c       Convert Conductances to isotropic Transmissivity arrays & Vertical Anisotropy ratios 
c  		Read LITHOLOGY, HydK, Ss, & Sy table
c  		Read reference X, Y and rotation  from Upper, Left corner
c  		Read Well location table --- X, Y & Screens   -- Multiple Tops & bottoms OK  -- Even depth entry required.
c  		Define each aquifer test site and specify,
c  					SITENAME, Pumping Well, Discharge, Test Duration, Depth-to-Water, WellRadius, AnnularRadius,
c  									  Number-of-Columns, Radial Extent, and Vertical Cell Thickness
c  					Lithology is provided as a table of depth and lith-intervals.
c  					    Top & bottom of lith table defines vertical extent of supporting radial model
c  		Optionally extract simulated drawdowns at observation locations in the supporting radial models
c  		Create all supporting radial models  w/ a master batch file to execute all.
c  		Supporting flow models create
c  					A drawdown array at the end of the pumping period/aquifer test.
c  					Drawdown time series at each observation well associated w/ an aquifer test.
c      Batch file executed after running T-COMP_Create.
c  					Store drawdown array for each aquifer test.
c  					Store drawdown time series, 1 file for each aquifer test.
c  		AquiferTestNAME w/ no spaces or other delimiters goes into a 32 character field.
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C1------USE package modules from MODFLOW.
      USE GLOBAL
      USE TcompReduceMODULE
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      character*5  sNOR
      character*5  sNOL
      character*5  sVAR
      CHARACTER*64 sSubMini
      CHARACTER*64 sWelNAME(nMAXwell)
      CHARACTER*64 sLITH(nMAXlith)
      CHARACTER*200 FNAME
      CHARACTER*2048 sTxT
      real rLITHprop(nMAXlith,4)
      real rWelINT(nMAXint)
      real rn(lrna)
      real rWelX(nMAXwell)
      real rWelY(nMAXwell)
      real rLAYfrac
      real rDDthresh
      real rRADmin 
      real rRADmax, rQfrac
      real rDEFsy 
      real rDEFss 
      real rDEFrHV
      INTEGER nWelINT(nMAXwell)
      INTEGER nWells
      INTEGER nZmini
      INTEGER nLITH
      INTEGER ierr
      INTEGER io
      INTEGER i
      LOGICAL(4) bYN
C1------USE package modules from T-COMP routines
C
      IGRID=1
      ioq = 95
      io = 107
      write(*,'(A)') ' Enter Output file for echoing checks' 
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
      if(ki.lt.1) sTxT = 'OUTPUT_T-Comp_CreateHISTORY.txt'
      open(io+10, file=sTxT)
C
C  Get MODFLOW STUFF 
      FNAME = '                         '
      call MF_DimNcond(IGRID,FNAME) 
C ___________________  START doing my stuff ______________________________________
c
c  Convert CC&CR to a node centered T array, Convert CV to Kz, and Calculate Kz/Kh 
      call Cond2Tran(IGRID) 
c
   5  write(*,'(A)') ' Enter instruction file for creating MiniModels' 
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 5
c      if(ki.lt.1) sTxT = '00_PartUP_INPUT_Tcomp.txt'
c      if(ki.lt.1) sTxT = '00_DVRFSTest_INPUT_Tcomp.txt'
      if(ki.lt.1) sTxT = '00_PMOV_INPUT_Tcomp.txt'
      open(io, file=sTxT)
      call DelimSET(io, sNOL, sNOR, sVAR )
c  Read info that ties regional model to world so mini models can be sampled.
      call LocalINFO(io,io+10,sNOL,sNOR,sVAR, nLITH,sLITH,rLITHprop, 
     +      nWells,sWelNAME,rWelX,rWelY, nWelINT,rWelINT,
     +      rDDthresh,rRADmin,rRADmax,rDEFsy,rDEFss,rDEFrHV,sSubMini,
     +      rQfrac,rLAYfrac)
c  Read all well locations & constructions to be considered. 
c      Build Mini MODFLOW models from templates 
       call MiniBUILD(io,io+10,sNOL,sNOR,sVAR, nLITH,sLITH,rLITHprop, 
     +                   nWells,sWelNAME,rWelX,rWelY, nWelINT,rWelINT,
     +                   rDEFsy,rDEFss,rDEFrHV,sSubMini)    
c
      close(io+1)
      close(io)
c  Echo conductances CC, CR, CV  for general check
c      nlay = 0
      do K = nlay+1,nlay
        sTxT = 'OUTPUT_Kh-LayerXXX.array.txt'
        ki = index(sTxT,'XXX')
        write(sTxT(ki:ki+2),'(i3.3)') k
        open(ioq,file=sTxT)
        sTxT = 'OUTPUT_Thk-LayerXXX.array.txt'
        ki = index(sTxT,'XXX')
        write(sTxT(ki:ki+2),'(i3.3)') k
        open(ioq+1,file=sTxT)
        sTxT = 'OUTPUT_Kv-LayerXXX.array.txt'
        ki = index(sTxT,'XXX')
        write(sTxT(ki:ki+2),'(i3.3)') k
        open(ioq+2,file=sTxT)
        sTxT = 'OUTPUT_KvKh-LayerXXX.array.txt'
        ki = index(sTxT,'XXX')
        write(sTxT(ki:ki+2),'(i3.3)') k
        open(ioq+3,file=sTxT)
c        write(iout,'(A,I10,A15)')' Layer:', k, sTxT
        do J = 1,nrow
          write(ioq,'(500G12.5)') (rTbar(i,j,k)/rThk(i,j,k),i=1,ncol)
          write(ioq+1,'(500G12.5)') (rThk(i,j,k),i=1,ncol)
          write(ioq+2,'(500G12.5)') (rKvBar(i,j,k),i=1,ncol)
          write(ioq+3,'(500G12.5)') (rKvKh(i,j,k),i=1,ncol)
        enddo
        close(ioq+3)
        close(ioq+2)
        close(ioq+1)
        close(ioq)
      enddo
C ___________________  Stop doing my stuff ______________________________________
C
C9------CLOSE FILES AND DEALLOCATE MEMORY. 
      call MF_DumpMemory(IGRID)
c---------------------------      
      CALL USTOP(' ')
      END
C ______________________________________________________________________________________________
c
