C     *****************************************************************
C     MAIN CODE FOR T-COMP_Extract
C     ******************************************************************
c Program  T-COMP_Extract.f   ---  Identifies contributing T nodes and assigns weights to each node for all aquifer tests    
c
c    Read regional MODFLOW files. 
c    Extract regional gridding. 
c    Read reference X, Y and rotation  from Upper, Left corner
c    Aquifer-test site identifier is keyed to stored drawdown array names   
c    Read each local MODFLOW file set
c          Extract each local grid to intersect local drawdowns on regional grid. 
c          Identify affected nodes in regional model and assicoate fractions for computing T. 
c    Write output for T-COMP_Simulated.f in the format,   --  This is the transmissivity factors file. 
c
c        AquiferTestNAME     !!  AquiferTestNAME w/ no spaces or other delimiters goes into a 32 character field.
c        Nnodes              !!  number of nodes to be compared 
c        Node-001     0.002
c        Node-001     0.032
c            .         .  
c            .         .  
c        Node-Nnodes  0.005    !!   All fractions must sum to 1  
c    Generic name for output RegionalNAME_TranFACTORS.txt     
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C1------USE package modules from MODFLOW.
      USE GLOBAL
      USE GWFBCFMODULE,ONLY:IBCFCB
      use GWFLPFMODULE,ONLY:ILPFCB
      USE GWFHUFMODULE,ONLY:IHUFCB 
c      
      USE TcompReduceMODULE
      USE IFPORT
c
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      character*5  sNOR
      character*5  sNOL
      character*5  sVAR
      CHARACTER*64 sSubMini
      CHARACTER*64 sWelNAME(nMAXwell)
      CHARACTER*64 sLITH(nMAXlith)
      CHARACTER*80 sGHBtemp
      CHARACTER*200 FNAME
      CHARACTER*2048 sTxT
      real rLITHprop(nMAXlith,4)
      real rWelINT(nMAXint)
      real rn(lrna)
      real rWelX(nMAXwell)
      real rWelY(nMAXwell)
      real rDDthresh
      real rLAYfrac
      real rRADmin 
      real rRADmax
      real rQfrac 
      INTEGER nWelINT(nMAXwell)
      INTEGER nWells
      INTEGER nZmini
      INTEGER nLITH
      INTEGER ierr
      INTEGER io, ioEcho
      INTEGER i
      LOGICAL(4) bYN
C1------USE package modules from T-COMP routines
C
      IGRID = 1
      ioq = 95
C
C  Get MODFLOW STUFF 
      Fname = ''
      call MF_DimNcond(IGRID,Fname) 
C ___________________  START doing my stuff ______________________________________
c
c  Convert CC&CR to a node centered T array, Convert CV to Kz, and Calculate Kz/Kh 
      call Cond2Tran(IGRID) 
c
      io = 5
      ioEcho = io+10
      write(*,'(A)') ' Enter Output file for echoing checks' 
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
      if(ki.lt.1) sTxT = 'OUTPUT_T-Comp_ExtractHISTORY.txt'
      open(ioEcho, file=sTxT)
c      
   5  write(*,'(A)') ' Enter MiniModel instruction file ....' 
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 5
      if(ki.lt.1) sTxT = '03_INPUT_PM-SS.Tcomp.txt'
      open(io, file=sTxT)
      call DelimSET(io, sNOL, sNOR, sVAR )
c  Read info that ties regional model to world so mini models can be sampled.
      call LocalINFO(io,ioEcho,sNOL,sNOR,sVAR, nLITH,sLITH,rLITHprop, 
     +      nWells,sWelNAME,rWelX,rWelY, nWelINT,rWelINT,
     +      rDDthresh,rRADmin,rRADmax,rDEFsy,rDEFss,rDEFrHV,sSubMini,
     +      rQfrac,rLAYfrac)
      close(io)
c Query and create node, fraction, volume file for each site      
  15  write(*,'(A)') ' Enter name of node-fraction OUTPUT file ....'
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 15
      if(ki.lt.1) sTxT = 'OUTPUT_Tcomp_Node-Fractions.txt'
      open(io, file=trim(sTxT))
      sTxT = 'ZZZ_'//sTxT
      open(io+1, file=trim(sTxT))
      sTxT(1:4) = 'RAD_'
      open(io+2, file=trim(sTxT))
      sTxT(1:4) = 'GHB_'
      open(io+4, file=trim(sTxT))
      write(io+4,'(2i10.9)') 99999, 90
      sGHBtemp = 't.GHB.txt' 
      open(io+3, file=trim(sGHBtemp))
c  25  write(*,'(A)') ' Enter name of PEST observation subfile ...'
c      read(*,'(A)') sTxT
c      call TrimALL(sTxT)
c      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 25
c      if(ki.lt.1) sTxT = 'OUTPUT_Tcomp_PEST-Measurements.txt'
c      open(io-1, file=trim(sTxT))
c  Query results of each Mini MODFLOW model to define integration volume. c
      call MiniMFER(rDDthresh,rRADmin,rRADmax, nWells,sWelNAME,
     +         rWelX,rWelY,iGRID, io,ioEcho, sSubMini,rQfrac,rLAYfrac) 
c
      write(io+2,'(a)') '<end>'
      close(io+4)
      close(io+3)
      close(io+2)
      close(io+1)
      close(ioEcho)
      close(io)
c 
c      io = DELFILESQQ(trim(sGHBtemp))
c
C ___________________  Stop doing my stuff ______________________________________
C
C9------CLOSE FILES AND DEALLOCATE MEMORY. 
      call MF_DumpMemory(IGRID)
c---------------------------      
      CALL USTOP(' ')
      END
C ______________________________________________________________________________________________
c
      subroutine MiniMFER(rDDthresh,rRADmin,rRADmax,nWells,sWelNAME, 
     +        rWelX,rWelY, iGRID, io,ioEcho, sSubMini,rQfrac,rLAYfrac)
      USE GLOBAL
      USE GWFBASMODULE,ONLY:PERTIM,TOTIM,IHEDUN
      USE TcompReduceMODULE
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      CHARACTER*1  sDEL
      CHARACTER*1  sCR
      CHARACTER*64 sSubMini
      CHARACTER*64 sSITE
      CHARACTER*64 sWelNAME(nMAXwell)
      CHARACTER*200 FNAME
      CHARACTER*2048 sTxT
      real rn(lrna)
      real rRADedge(nMAXminiR)
      real rZedge(nMAXminiZ)
      real rRADS(nMAXminiZ)
      real rQS(nMAXminiZ)
      real rKvol(nMAXminiZ)
      real rRegAREA(nMAXminiZ), rRegTHK(nMAXminiZ), rRegZfrac(nMAXminiZ)
      real rRegQ(nMAXminiZ), rRegQmax
      real rWelX(nMAXwell)
      real rWelY(nMAXwell)
      real xFind
      real yFind
      real rDDthresh
      real rLAYfrac, rZfrac
      real rRADmin, rAREAmin, rAREAmax 
      real rRADmax, rEXT 
      real rQfrac 
      real rSIMmax
      real rAREA, rTOP, rBOT, rBz, rFRAC, rFracLAY, rBall 
      INTEGER nWells
      INTEGER nZmini
      INTEGER nRmini, nCntGHB
      INTEGER ierr
      INTEGER io, ioEcho
      INTEGER m, i,j, kMM, kREG, ii1,jj1, ii2,jj2
      INTEGER ii,jj, kMM1, kMM2, nNODES
      INTEGER ki
      INTEGER nNodePwell(nMAXwell)
      LOGICAL(4) bYN
c
      sCR = char(10)
      sDEL = '\' 
      rAREAmin = 1.0 
      sTxT = '.'//sDEL//trim(sSubMini)  
      bYN = CHANGEDIRQQ(trim(sTxT))
      nCntGHB = 0 
C  Get MODFLOW STUFF 
      do m = 1, nWells                           !!   Loop on each T observation
        nNodePwell(m) = 0
        call SGWF2BAS7PNT(IGRID)
        call iNODEreg(rWelX(m), rWelY(m), 1, ii, jj, 1)
        sSITE = trim(sWelNAME(m))
        write(ioEcho,*) 
        write(ioEcho,'(a17,a32)') 
     +    ' Processing site:',sSITE
c          ----+----1----+----2----+----3----+----4----+----5
        if( ii.eq.0 .or. jj.eq.0 ) goto 25
c         
        rZedge(1) = botm(ii,jj,0)
        Fname = 'INput_'//trim(sSITE)//'.NAM.txt'
        sTxT = '.'//sDEL//trim(sSITE)//sDEL//Fname
        INQUIRE (file=sTxT,exist=bYN)      
        if( .not. bYN ) goto 25
c        
        sTxT = '.'//sDEL//trim(sSITE)
        bYN = CHANGEDIRQQ(trim(sTxT))
        call MF_DimNcond(IGRID+1, Fname)
c  Extract Mini Model coords before passing back regional MODFLOW model        
        nRmini = NCOL
        call DelX2edge(rRADedge)
        nZmini = NROW
        call DelZ2edge(rZedge)
        rSIMmax = rRadByDD(rDDthresh,rRADmax, rRADedge,
     +                     rZedge, rRADS, rQS)   !!   rRADS is extent of investigation based on Sim. DD
        write(ioEcho,'(a30,g16.5)')
     +    ' Maximum radius investigated =', rSIMmax
c          ----+----1----+----2----+----3----+----4----+----5
        write(io+1,'(a)') trim(sSITE)
        write(io+1,'(3G14.6)')
     +     ((rZedge(j+1)+rZedge(j))/2., rRADS(j), rQS(j), j=1,nZmini)
        write(io+2,'(a)') trim(sSITE)
        write(io+2,'(2G14.6)') 
     +   (rWelX(nWells+1)+rWelX(m)+rSIMmax*cos(j*3.14159/18.), 
     +    rWelY(nWells+1)+rWelY(m)+rSIMmax*sin(j*3.14159/18.),j=1,37)
c   Pass regional model back and set bounds for scanning array
        call SGWF2BAS7PNT(IGRID)
c  Accumulate Thickness by regional model layer 
        call iNODEreg(rWelX(m),rWelY(m), 0, i, j, 1 )
        kMM2 = 0
        do kREG = 1, Nlay                              !   Define fine-scale layer extents from regional layer
          rRegQ(kREG)   = 0.000000000000000000
          rRegTHK(kREG) = 0.000000000000000000
          kMM1 = iMATCHr(botm(i,j,kREG-1),rZedge,nZmini, -1)
          if( kMM1.eq.kMM2 ) kMM1 = kMM1+1 
          kMM2 = iMATCHr(botm(i,j,kREG),rZedge,nZmini, -1)
          if( kMM2.ge.kMM1 .and. kMM1.gt.0 ) then
            do kMM = kMM1, kMM2 
              rEXT = rRADS(kMM)
              rTOP = rZedge(kMM)
              if( kMM.eq.kMM1 ) rTOP = botm(i,j,kREG-1)
              rBOT = rZedge(kMM+1) 
              if( kMM.eq.kMM2 ) rBOT = botm(i,j,kREG) 
              rBz = rTOP - rBOT
              rRegQ(kREG) = rRegQ(kREG) + rQS(kMM)
              if( rEXT .gt. rRADmin ) then
                rRegTHK(kREG) = rRegTHK(kREG) + rBz
              endif
            enddo       !   Mini model sub loop
            rBz = botm(i,j,kREG-1) - botm(i,j,kREG) 
            rRegZfrac(kREG) = rRegTHK(kREG) / rBz
          endif
        enddo         !   Regional model layers 
c  Define bounding coordinates of area search 
        xFind = rWelX(m) - rSIMmax
        yFind = rWelY(m) - rSIMmax
        call iNODEreg(xFind,yFind, 0, ii1, jj1, 1 )
        xFind = rWelX(m) + rSIMmax
        yFind = rWelY(m) + rSIMmax
        call iNODEreg(xFind,yFind, 0, ii2, jj2, 1 )
c  Redefine limits if 1 pair in domain
        if( ii1.eq.0 .and. ii2.gt.0 ) then
          xFind = rWelX(m) - rSIMmax
          yFind = rWelY(m) - rSIMmax
          call iNODEreg(xFind,yFind, 0, ii1, jj1, 0 )
        endif
        if( ii1.gt.0 .and. ii2.eq.0 ) then
          xFind = rWelX(m) + rSIMmax
          yFind = rWelY(m) + rSIMmax
          call iNODEreg(xFind,yFind, 0, ii2, jj2, 0 )
        endif
c   Initialize volume accumulators at the regional model grid scale -- Use existing MF arrays       
c     HOLD = volume term;  BUFF = thicknesses 
        do i = 1,NCOL                                   !   Loop for areal weighting of T 
          do j= 1,NROW                                    !   Loop for areal weighting of T 
            do kREG = 1, Nlay                              !   Initialize thickness w/ detected drawdown accumulators
              HOLD(i,j,kREG) = 0.000
              BUFF(i,j,kREG) = 0.000
              DDREF(i,j,kREG) = 0.000
            enddo
          enddo
        enddo
c  Accumulate terms        
        do i = ii1,ii2                                     !    Loop for areal weighting of T 
          do j= jj1,jj2, -1                                    !   Loop for areal weighting of T 
            kMM2 = 0
            do kREG = 1, Nlay                              !   Define fine-scale layer extents from regional layer
              if( ibound(i, j,kREG).ne.0 ) then
                kMM1 = iMATCHr(botm(i,j,kREG-1),rZedge,nZmini, -1)
                if( kMM1.eq.kMM2 ) kMM1 = kMM1+1 
                kMM2 = iMATCHr(botm(i,j,kREG),rZedge,nZmini, -1)
                if( kMM2.gt.kMM1 ) kMM2 = kMM2 - 1
                if( kMM2.ge.kMM1 .and. kMM1.gt.0 ) then
                  do kMM = kMM1, kMM2 
                    rEXT = rRADS(kMM)
                    rTOP = rZedge(kMM)
                    if( kMM.eq.kMM1 ) rTOP = botm(i,j,kREG-1)
                    rBOT = rZedge(kMM+1) 
                    if( kMM.eq.kMM2 ) rBOT = botm(i,j,kREG) 
                    rBz = rTOP - rBOT
                    if( rEXT .gt. rRADmin ) then
                      rAREA = dAreaINTERSECT(i,j,rWelX(m),rWelY(m),rEXT)
                      HOLD(i,j,kREG) = HOLD(i,j,kREG) + rAREA * rBz
                      BUFF(i,j,kREG) = BUFF(i,j,kREG) + rBz
                    endif
                  enddo       !   Mini model sub loop
                endif
              endif
            enddo         !   Regional model layers 
          enddo           !   Regional model rows 
        enddo             !   Regional model columns 
c   Convert to areas, ADD total area per regional layer, and count nodes of interest  
        nNODES = 0
        do kREG = 1, Nlay               
          rRegQmax = 0.00000
          rRegAREA(kREG) = 0.000000000000000000
          do i = ii1,ii2                    
            do j= jj1,jj2, -1               
              if( ibound(i, j,kREG).ne.0 ) then
                rBz = BUFF(i,j,kREG) 
                if( rBz.gt.1.0E-10 ) HOLD(i,j,kREG) = HOLD(i,j,kREG)/rBz
                if( HOLD(i,j,kREG).gt. rAREAmin ) then
                  rRegAREA(kREG) = rRegAREA(kREG) + HOLD(i,j,kREG)
                  nNODES = nNODES + 1
                endif  
              endif  
            enddo           !   Regional model rows 
          enddo             !   Regional model columns 
        enddo         !   Regional model layers 
c   Scan and write nodes & fractions 
        if( nNODES.gt.0 ) then
          write(io,'(a)') trim(sSITE)
          write(io,'(i10.10,2x,a11,19x,a4,8x,a13,4x,a13,4x,a11)')nNODES,
     +     '!! Fraction',
     +     'Area',
     +     'Vert.Fraction',
     +     'Frac.Max.Area',
     +     'Cell Trans.'
c           ----+----1----+----2----+----3----+----4----+----5
          write(ioEcho,'(a35,i10)')
     +    ' Regional model nodes intersected =', nNODES
c          ----+----1----+----2----+----3----+----4----+----5
          rRegQmax = 0.00000
          rAREAmax = 0.000
          do kREG = 1, Nlay                   
            if( rRegQmax .lt. rRegQ(kREG) )   rRegQmax = rRegQ(kREG)
            if( rAREAmax .lt. rRegAREA(kREG)) rAREAmax = rRegAREA(kREG)
          enddo         !   Regional model layers 
c        
          write(ioEcho,'(a38,g16.5)')
     +    ' Maximum area intersected =          ', rAREAmax
          write(ioEcho,'(a38,g16.5)')
     +    ' Maximum Regional Flow intersected = ', rRegQmax
c          ----+----1----+----2----+----3----+----4----+----5
c        
          do kREG = 1, Nlay
            rFracLAY = rRegQ(kREG)/rRegQmax / rQfrac
            if( rFracLAY.gt.1.0000 ) then
              rFracLAY = 1.000
            else
              rFracLAY = 0.000
            endif  
            do i = ii1,ii2                    
              do j= jj1,jj2, -1               
                if( HOLD(i,j,kREG).gt. rAREAmin ) then
                  rZfrac = rRegZfrac(kREG)
                  if( rZfrac.gt.rLAYfrac ) rZfrac = 1.000
                  rFRAC = rZfrac*HOLD(i,j,kREG)/rRegAREA(kREG)
                  rFRAC = rFRAC*rFracLAY  
                  nNODES = (kREG-1)*NROW*NCOL + (j-1)*NCOL + i
                  write(io,'(1x,i9,g16.8,5x,a2,5x,8g16.8)') nNODES, 
     +            rFRAC,'!!', HOLD(i,j,kREG),rZfrac,rFracLAY,
     +                    rTbar(i,j,kREG)
c         GHB like output to see rough, digitized extent  
                  if( rFRAC.gt.1.0E-6 ) then  
                    write(io+3,'(3i10,2F10.3,8x,a32)')
     +                    kREG,j,i, 1.0,1.0,trim(sSITE)
                    nCntGHB = nCntGHB + 1
                    nNodePwell(m) = nNodePwell(m) + 1
                  endif  
                endif  
              enddo           !   Regional model rows 
            enddo             !   Regional model columns 
          enddo         !   Regional model layers 
        endif    !! nNODES check block
c
c   Build and write measurement file 
c        
c        write(io-1,'()') sTxT, rLOGtran, rWGT, sCLASS 
c Me___TT53R   4731.0      1.000000      WL_OBS        
c
        call MF_DumpMemory(IGRID+1)
        bYN = CHANGEDIRQQ('..')
  25    continue        
      end do         !!  loop on wells
c  
c   tart-up GHB for all in first SP
      write(io+4,'(i10.10)') nCntGHB
      do j = 1,2
        rewind(io+3)
        do m = 1, nWells                           !!   Loop on each T observation
          kREG = nNodePwell(m)
          if( j.gt.1 .and. kREG.gt.0 ) write(io+4,'(i10.10)') kREG
          do kMM2 = 1, kREG
            read(io+3,'(a)') sTxT
            write(io+4,'(a90)') sTxT
          enddo 
        enddo
      enddo
c   Pad Bogus GHB to keep ModelViewer from crashing
      write(io+4,'(i10)')(0,i=1,100)
c
      return 
      end     
c ---------------------------------------------------------------------------------
      subroutine DelX2edge(rRADedge)
      USE GLOBAL
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      real rRADedge(nMAXminiR)
      integer i
c
      rRADedge(1) = 0.000
      do i = 1, NCOL
        rRADedge(i+1) = rRADedge(i) + delr(i)
      end do      
c
      return
      end      
c ---------------------------------------------------------------------------------
      subroutine DelZ2edge(rZedge)
      USE GLOBAL
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      real rZedge(nMAXminiZ)
      integer i
      integer ii, jj
c
      do i = 1, NROW
        rZedge(i+1) = rZedge(i) - delc(i)
      end do      
c
      return
      end      
c ---------------------------------------------------------------------------------
      real function rRadByDD(rDDthresh,rRADmax, rRADedge, 
     +                       rZedge, rRADS, rQS)
      USE GLOBAL
      USE GWFBASMODULE,ONLY:PERTIM,TOTIM,IHEDUN
      USE GWFBCFMODULE,ONLY:IBCFCB
      use GWFLPFMODULE,ONLY:ILPFCB
      USE GWFHUFMODULE,ONLY:IHUFCB 
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      character*16 TexT
      real rRADedge(nMAXminiR)
      real rn(ln1)
      real rZedge(nMAXminiZ)
      real rRADS(nMAXminiZ)
      real rQS(nMAXminiZ)
      real rDDthresh
      real rSIM, rQsim
      real rDD, rDD1, rFRAC 
      integer i, ioCBC
      integer j, iX
c
      IF(IUNIT(1).GT.0) then 
        ioCBC = IBCFCB
      elseif(IUNIT(23).GT.0) then
        ioCBC = ILPFCB
      elseif(IUNIT(37).GT.0) then
        ioCBC = IHUFCB
      else
      endif
      TexT = 'Keep Going        ' 
      DO WHILE (TexT .NE. 'FLOW RIGHT FACE ')     
        READ(ioCBC) KSTP,KPER,TEXT,ix,iy,kz
        READ(ioCBC) (((rhs(i,j,k),I=1,ix),j=1,iy),k=1,kz)
      enddo  
c
c      write(15,*) TexT
c      do j=1,iy
c        write(15,'(200E15.7)') (rhs(i,j,1),I=1,ix)
c      end do  
c
      read(ihedun)  KSTP,KPER,PERTIM,TOTIM,TEXT,ix,iy,kz
      do j = 1,NROW
        read(ihedun) (buff(i,j,1), i=1,NCOL)
      end do       
c
      rRadByDD = 0.0
      rQS(NROW+1) = 0.0
      do j = 1, NROW
        do i = 3,NCOL
          rn(i-2) = buff(i,j,1)
        enddo
        iX = Ncol-2
	    i = iMATCHr(rDDthresh, rn, iX, -1)
	    if( i.lt.1 ) then
            rSIM = rRADedge(i+2)
	      rQsim = rhs(i+2,j,1)
	    else
          rFRAC = (rDDthresh - rn(i)) / (rn(i+1)-rn(i))
          rSIM = rRADedge(i+2) + rFRAC*( rRADedge(i+3)-rRADedge(i+2))
          rQsim = rhs(i+2,j,1) + rFRAC*( rhs(i+3,j,1) -rhs(i+2,j,1))
	    endif
        if( rSIM.gt.rRADmax ) rSIM = rRADmax 
        if( rQsim.gt.rQS(NROW+1) ) rQS(NROW+1) = rQsim 
        rRADS(j) = rSIM
        rQS(j)   = rQsim
        if( rSIM.gt.rRadByDD ) rRadByDD = rSIM 
      end do      
      if( rRADS(NROW).gt.1.0 ) rRADS(NROW)= rDDthresh
c
      return
      end      
c ---------------------------------------------------------------------------------
