C     ******************************************************************
C     COMMON subroutines for all T-COMP programs 
C     ******************************************************************
c  Program  T-COMP_Create.f   ---   Creates comparable radial models and batch file to run all.
c
c  		Read LITHOLOGY, HydK, Ss, & Sy table
c  		Read reference X, Y and rotation  from Upper, Left corner
c  		Read Well location table --- X, Y & Screens   -- Multiple Tops & bottoms OK  -- Even depth entry required.
c  		Define each aquifer test site and specify,
c  					SITENAME, Pumping Well, Discharge, Test Duration, Depth-to-Water, WellRadius, AnnularRadius,
c  									  Number-of-Columns, Radial Extent, and Vertical Cell Thickness
c  					Lithology is provided as a table of depth and lith-intervals.
c  					    Top & bottom of lith table defines vertical extent of supporting radial model
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
c ---------------------------------------------------------------------------------
      real function rGET(io,sNOL,sNOR,ierr)
      include 'T-COMP_Mini.inc'
      character*5   sNOR
      character*5   sNOL
      CHARACTER*2048 sTxT
      INTEGER ierr
      INTEGER io
      real rn(lrna)
c
      rGET = -1.0E30 
      call ncread(io,sTxT,sNOL,sNOR,ierr)
      if( ierr.eq.0 ) then
        sTxT = sTxT(index(sTxT,'=')+1:len(sTxT))
        call qread(rn,1,sTxT,ierr)
        if( ierr.eq.0 ) rGET = rn(1)
      endif  
c
      return
      end
c ---------------------------------------------------------------------------------
      subroutine sGET(io,sNOL,sNOR,ierr, sTAG, sTxOUT)
      CHARACTER sTxOUT*(*)
      include 'T-COMP_Mini.inc'
      character*5   sNOR
      character*5   sNOL
      character*(*)  sTAG
      CHARACTER*2048 sTxT
      INTEGER ierr
      INTEGER io
      integer ki
      integer n
c
      sTxOUT = ""
      
      call ncread(io,sTxT,sNOL,sNOR,ierr)
      if( ierr.eq.0 ) then
        n = Len( trim(sTAG) )  
        ki = index(sTxT,sTAG(1:n))
        if( ki.gt.0 ) then
          sTxOUT = sTxT(n+ki:Len(sTxT))
        else
          backspace(io)
        endif
      endif  
c
      return
      end
c ---------------------------------------------------------------------------------
      subroutine DelimSET(io, sNOL, sNOR, sVAR )
c
      include 'T-COMP_Mini.inc'
      real rn(lrna)
      character*5   sNOR
      character*5   sNOL
      character*5   sVAR
      character*2048 txt
      integer io 
      iMAX = 5
      sNOL = '.'
      sNOR = '!'
      sVAR = '.'
c
c   Check if delimiters are to be redefined       
      do i = 1, 3
        read(io,'(a)',end=10) txt
        call UPCASE(txt)
        call tab2space(txt)
        k1 = index(txt,'NO-LINE_DELIMITER:')
        if( k1.gt.0 ) then
          k1 = k1 + Len('NO-LINE_DELIMITER:') 
	    sNOL = txt(k1:k1+iMAX-1) 
          k1 = index(sNOL,' ')
          if( k1.gt.0 ) sNOL(k1:iMAX) = '      '
	  endif
        k1 = index(txt,'NO-RIGHT_DELIMITER:')
        if( k1.gt.0 ) then
          k1 = k1 + Len('NO-RIGHT_DELIMITER:') 
	    sNOR = txt(k1:k1+iMAX-1)  
          k1 = index(sNOR,' ')
          if( k1.gt.0 ) sNOR(k1:iMAX) = '      '
	  endif
        k1 = index(txt,'VARIABLE_DELIMITER:')
        if( k1.gt.0 ) then
          k1 = k1 + Len('VARIABLE_DELIMITER:') 
	    sVAR = txt(k1:k1+iMAX-1)  
          k1 = index(sVAR,' ')
          if( k1.gt.0 ) sVAR(k1:iMAX) = '      '
	  endif
      enddo  
  10  rewind(io)
c
      return
      end  
c ---------------------------------------------------------------------------------
c
      subroutine sSUBstring(sTEMP,sNEW, sVAR, sTxOUT)
      CHARACTER*2048 sTEMP
      CHARACTER*2048 sWORK
      CHARACTER*2048 sADD
      character*64 sNEW
      CHARACTER*(*) sTxOUT
      character*5  sVAR
      integer iMax
      integer ki
      integer ki2
      integer n
c
      sWORK = sTEMP
      n = Len(trim(sVAR))
      ki = index(sWORK,sVAR(1:n))
      if( ki.gt.0 ) then
        sWORK(ki:ki) = ' '
        sADD = sTEMP(1:ki-1) // sNEW
        ke = index(sWORK,sVAR(1:n))
        n = Len(trim(sWORK))
        if( ke.gt.0 ) then
          sTxOUT = trim(sADD) // sTEMP(ke+1:n)
        else  
          sTxOUT = trim(sNEW) // sTEMP(ki:n) 
        endif  
      else  
        sTxOUT = ""
      endif  
c
      return
      end
c ---------------------------------------------------------------------------------
	integer function iMATCHs(sFIND, sARRlook, nLOOK)
      include 'T-COMP_Mini.inc'
      CHARACTER*64 sFIND
      CHARACTER*64 sARRlook(ln1)
      integer nLOOK
      integer i
      integer n
c
      iMATCHs = 0
      i = 0
      n = len(trim(sFIND))
      do while (i < nLOOK) 
        i = i+1
        if( sFIND(1:n).eq.sARRlook(i)(1:n) ) then
          iMATCHs = i
          i = nLOOK 
        endif
      end do
c
      return
	end
c ---------------------------------------------------------------------------------
	integer function iMATCHr(rFIND, rARRlook, nLOOK, iDIRin)
      include 'T-COMP_Mini.inc'
      integer nLOOK
      integer i
      integer iDIR
      integer iDIRin
      real rFIND
      real rARRlook(ln1)
c
      if( iDIRin.lt.0 ) then 
        iDIR = -1
      else  
        iDIR = 1
      endif
c     
      if( rFIND*iDIR .ge. rARRlook(nLOOK)*iDIR ) then
        iMATCHr = nLOOK
      else
        iMATCHr = 0
        i = 0
        do while (i < nLOOK) 
          i = i+1
          if( rFIND*iDIR .lt. rARRlook(i)*iDIR ) then
            iMATCHr = i-1
            i = nLOOK+1 
          endif
        end do
      endif
c
      return
	end
c ---------------------------------------------------------------------------------
c
      subroutine iNODEreg(xFind,yFind, iKz, ii, jj, iBtest )
      use global
      integer i  
      integer iBtest  
      integer ii 
      integer j 
      integer jj 
      integer k
      real rADD 
c
      ii = 0
      rADD = 0.
      do i = 1, ncol
        if( xFind.gt.rADD .AND. xFind.le.rADD+delr(i) ) ii = i
        rADD = rADD + delr(i)
      end do
c
      jj = 0
      if( ii.gt.0 ) then
        rADD = 0.
        do j = 1, nrow
          if( yFind.lt.rADD .AND. yFind.ge.rADD-delc(j) ) jj = j
          rADD = rADD - delc(j)
        end do
      endif  
c 
      if( ii.gt.0 .and. jj.gt.0 .and. iBtest.gt.0 ) then
        k = iKz
        if( k.lt.1 ) k = 1
        if( ibound(ii,jj,k).eq.0 ) then
          ii = 0
          jj = 0
        endif  
      endif   
c
      return
      end
c ---------------------------------------------------------------------------------
c
      real function rFRCmlt(rBASE,rEXTENT,iCNT)
      real rA
      real rBASE
      real rCLOSE
      real rEXTENT
      real rFRC
      real rRAT
      real rROOT
      integer i
      integer iCNT
      data rCLOSE /1.E-8/
c
c   Make a first cut at multiplier ...................
c
      rA = 1.00000000000
      rFRCmlt = rA
      rRAT = rEXTENT / rBASE
      if( iCNT .gt. rRAT ) return
c
      rROOT = 1. / iCNT
      rA  = alog(rRAT)/(iCNT*2)
      rA  = exp(rA)
c
      i = 0
      rFRC = 0.000
      do while( i.lt.99 .and. abs(rFRC - 1.).gt.rCLOSE ) 
        i   = i + 1
        rFRC = ( rRAT /((1.- rA**iCNT)/(1.-rA)))**rROOT
        rA   =  rA * rFRC
      enddo
      rFRCmlt = rA
c
      return
      end
c ---------------------------------------------------------------------------------
      real Function dAreaINTERSECT(iI, iJ,dXcen, dYcen, dRAD )
      USE TcompReduceMODULE
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      Integer iI, iJ
      real  dXcen, dYcen, dRAD 
      integer i1, j1, i2, j2
      Integer k, nNP, nNP1 
      real dDxy, dPI, dZERO, dMIN  
      real dXYpr(8, 2) 
      real dXp(lrna)
      real dYp(lrna)
      logical b1, b2, b3, b4
      real dNULL 
      dNULL = -999999999998.
      dZERO = 1.0E-20
      dPI = 3.141592654
c'
c ' test if circle is in square
      b1 = rXE(iI).le.(dXcen-dRAD) 
      b2 = rXE(iI + 1).gt.(dXcen+dRAD)
      b3 = rYE(iJ + 1).le.(dYcen-dRAD)
      b4 = rYE(iJ).gt.(dYcen+dRAD)
c      
      dMIN = Min(Abs(rXE(iI)-dXcen), Abs(rXE(iI + 1)-dXcen), 
     +        Abs(rYE(iJ)-dYcen), Abs(rYE(iJ + 1)-dYcen) )  
c     
      If( b1 .And. b2 .And. b3 .And. b4 )Then
        dAreaINTERSECT = dPI * dRAD ** 2
c'  Test if square does not intersect circle.
      ElseIf( dRAD.le.dMIN ) Then
        dAreaINTERSECT = 0.
      Else
c' Scan all 4 segments of each cell & assign 2 points to every segment that intersects
        nNP = 0
        do k = 0, 3
          j1 = Int(Mod(k, 4) / 2)
          i1 = mod((k + j1), 2)
          j2 = Int( Mod((k + 1),4) / 2)
          i2 = mod((k + 1 + j2), 2)
          Call dCircle_in_SEG(rXE(iI + i1), rYE(iJ + j1), rXE(iI + i2), 
     +       rYE(iJ + j2), dXcen, dYcen,dRAD, dXYpr(nNP+1,1), 
     +      dXYpr(nNP+1, 2), dXYpr(nNP+2, 1), dXYpr(nNP+2, 2)) 
          If( dXYpr(nNP+1, 1) .gt. dNULL )  nNP = nNP + 2
        end do          ! '  Next k
c'   Translate segments to continuous path of segments and arcs
        nNP1 = 0
        do k = 2,nNP,2
          i1 = mod( (k + 1), nNP )
          dXp(nNP1 + 1) = dXYpr(k - 1, 1)
          dXp(nNP1 + 2) = dXYpr(k, 1)
          dYp(nNP1 + 1) = dXYpr(k - 1, 2)
          dYp(nNP1 + 2) = dXYpr(k, 2)
          nNP1 = nNP1 + 2
          dDxy = (dXYpr(i1, 1)-dXYpr(k, 1))**2 + 
     +           (dXYpr(i1, 2)-dXYpr(k,2))**2
          If( dDxy.lt.dZERO )Then
c'           nNP1 = nNP1 - 1
          Else
            dXp(nNP1 + 1) = dXYpr(i1, 1)
            dYp(nNP1 + 1) = dXYpr(i1, 2)
            nNP1 = nNP1 + 1
c'        Add Arc sweeping call
c            Call dAddARC(dXcen, dYcen, dRAD, dXp, dYp, nNP1, 10.)
          End If
        end do                  !  ' Next k
c'  Compute area of polgon approximation
        dAreaINTERSECT = dPolyArea(dXp, dYp, nNP1)
c        write(7,*)' dAreaINTERSECT  =',dAreaINTERSECT , ii, iJ 
c        write(7,'(2g17.8)')( dxp(i),dYp(i),i=1,nNP1)
      End If
c'
      return
      end
c ---------------------------------------------------------------------------------
      real Function dPolyArea(Xlook, Ylook, NR) 
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      integer k, NR, Nval, nC
      real dAdd, Sum
      real Xlook(lrna), Ylook(lrna)
c
c    Area of polygon by SUM  x2*y1 - y2*x1
c    + areas denote a Clockwise curve
c    - areas denote a Counterclockwise curve
c'
      If( NR.gt.2 )Then
        Sum = 0.
        Xlook(NR + 1) = Xlook(1)
        Ylook(NR + 1) = Ylook(1)
c'
        do k = 1, NR
          dAdd = Xlook(k) * Ylook(k + 1) - Xlook(k + 1) * Ylook(k)
          Sum = Sum + dAdd
        end do  !  Next k
        dPolyArea = Abs(Sum) / 2.
      Else
        dPolyArea = 0.
      End If
c'
      return
      end
c ---------------------------------------------------------------------------------
      real function dIIf(bYN,d1,d2)
      logical bYN
      real d1,d2
c'
      if( bYN ) then
        diif = d1
      else
        diif = d2
      endif 
c'
      return
      end        
c ---------------------------------------------------------------------------------
      Subroutine dCircle_in_SEG(Dx1,Dy1, Dx2,Dy2, dXcen,dYcen,dRAD, 
     +                          dX1int,dY1int, dX2int,dY2int)
      real Dx1,Dy1, Dx2,Dy2
      real dXcen,dYcen,dRAD
      real dX1int,dY1int, dX2int,dY2int
      real dR1,dR2, dRad2
      real dDX1,dDY1, dDX2,dDY2, dADJ, dOPP
      Integer iPOLE
      logical bUpDown, bINcell
      real dNULL 
      dNULL = -999999999999.
c'
      dX1int = dNULL
      dY1int = dNULL
      dX2int = dNULL
      dY2int = dNULL
c'
      dDX1 = Dx1 - dXcen
      dDY1 = Dy1 - dYcen
      dR1 = (dDX1 ** 2 + dDY1 ** 2) ** 0.5
      dDX2 = Dx2 - dXcen
      dDY2 = Dy2 - dYcen
      dR2 = (dDX2 ** 2 + dDY2 ** 2) ** 0.5
c'
      bUpDown = Abs(dDX1 - dDX2) < 1E-20           !'  Segment up-down if true; side-to-side if false
      dADJ = dIIf(bUpDown, dDX1, dDY1)
      dADJ = abs(dADJ)
      If( bUpDown ) then 
        bINcell =  Min(Dy1, Dy2).lt.dYcen .And. dYcen.le.Max(Dy1, Dy2)
      else   
        bINcell =  Min(Dx1, Dx2).lt.dXcen .And. dXcen.le.Max(Dx1, Dx2)
      endif  
c'------------Segment lies OUTside circle and does not intersect ----------------
      If( Min(dR1, dR2, dIIf(bINcell, dADJ, dR2)) .gt. dRAD )Then
        dX1int = dNULL
        dY1int = dNULL
        dX2int = dNULL
        dY2int = dNULL
c'------------Segment lies inside circle and does not intersect ----------------
      ElseIf( Max(dR1, dR2, dADJ) .lt. dRAD )Then
        dX1int = Dx1
        dY1int = Dy1
        dX2int = Dx2
        dY2int = Dy2
      Else
c'------------Circle intersects once or twice, compute by corner ----------------
        dOPP = (dRAD ** 2 - dADJ ** 2) ** 0.5
        If( dR1 .le. dRAD )Then
          dX1int = Dx1
          dY1int = Dy1
        Else
          iPOLE = dIIf(bUpDown, dDY1 / Abs(dDY1), dDX1 / Abs(dDX1))
          dX1int = dIIf(bUpDown, Dx1, dXcen + dOPP * iPOLE)
          dY1int = dIIf(bUpDown, dYcen + dOPP * iPOLE, Dy1)
        End If
        If( dR2 .le. dRAD )Then
          dX2int = Dx2
          dY2int = Dy2
        Else
          iPOLE = dIIf(bUpDown, dDY2 / Abs(dDY2), dDX2 / Abs(dDX2))
          dX2int = dIIf(bUpDown, Dx2, dXcen + dOPP * iPOLE)
          dY2int = dIIf(bUpDown, dYcen + dOPP * iPOLE, Dy2)
        End If
      End If
c'
      return
      end
c ---------------------------------------------------------------------------------
      subroutine dAddARC(dXcen, dYcen, dRAD, dXp,dYp, nNP, dDEG)
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      real  dTheta1,dTheta, dThetaINC
      real dDX1,dDY1, dDX2,dDY2 As Double
      Integer i, iNPT, nNP
      real  dXcen, dYcen, dRAD, dDEG, dPI 
      real dXp(lrna), dYp(lrna)
      dPI = 3.141592654
c'
      dDX1 = dXp(nNP - 1) - dXcen
      dDY1 = dYp(nNP - 1) - dYcen
      dTheta1 = fang(dDX1, dDY1)
      dDX2 = dXp(nNP) - dXcen
      dDY2 = dYp(nNP) - dYcen
      dThetaINC =  v2rad(dDX1, dDY1, dDX2, dDY2) - 2 * dPI
      iNPT = Int(Abs(dThetaINC) / (0.017453293 * dDEG)) + 1   ! Deg arc segments
      dThetaINC = dThetaINC / iNPT
c'
      dXp(nNP + iNPT) = dXp(nNP)
      dYp(nNP + iNPT) = dYp(nNP)
      nNP = nNP - 1
      do i = 1, iNPT
        dTheta = dTheta1 + dThetaINC * i
        dXp(nNP + i) = dXcen + dRAD * Cos(dTheta)
        dYp(nNP + i) = dYcen + dRAD * Sin(dTheta)
      end do  !  Next i
      nNP = nNP + iNPT + 1
c'
      return
      end
