      SUBROUTINE MF_DimNcond(IGRID,FnamePASS)
C     ******************************************************************
C     MODFLOW 2005 stripped to subroutine for dimension and conductance calculations 
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C1------USE package modules.
      USE GLOBAL
      USE GWFBASMODULE
      USE GWFHUFMODULE, ONLY:IOHUFHDS,IOHUFFLWS
      INCLUDE 'openspec.inc'
C
C-------ASSIGN VERSION NUMBER AND DATE
      CHARACTER*40 VERSION
      CHARACTER*10 MFVNAM
      CHARACTER*16 sTxT
      PARAMETER (VERSION='1.8.00 12/18/2009')
      PARAMETER (MFVNAM='-2005')
C
      CHARACTER*80 HEADNG(2)
      CHARACTER*200 FNAME, FnamePASS
      INTEGER IBDT(8)
C
      CHARACTER*4 CUNIT(NIUNIT)
      DATA CUNIT/'BCF6', 'WEL ', 'DRN ', 'RIV ', 'EVT ', '    ', 'GHB ',  !  7
     &           'RCH ', 'SIP ', 'DE4 ', '    ', 'OC  ', 'PCG ', 'lmg ',  ! 14
     &           'gwt ', 'FHB ', 'RES ', 'STR ', 'IBS ', 'CHD ', 'HFB6',  ! 21
     &           'LAK ', 'LPF ', 'DIS ', '    ', 'PVAL', '    ', 'HOB ',  ! 28
     &           '    ', '    ', 'ZONE', 'MULT', 'DROB', 'RVOB', 'GBOB',  ! 35
     &           'STOB', 'HUF2', 'CHOB', 'ETS ', 'DRT ', '    ', 'GMG ',  ! 42
     &           'HYD ', 'SFR ', '    ', 'GAGE', 'LVDA', '    ', 'LMT6',  ! 49
     &           'MNW2', 'MNWI', 'MNW1', 'KDEP', 'SUB ', 'UZF ', 'gwm ',  ! 56
     &           'SWT ', 'cfp ', '    ', '    ', '    ', '    ', 'nrs ',  ! 63
     &           37*'    '/
C     ------------------------------------------------------------------
C
C2------WRITE BANNER TO SCREEN AND DEFINE CONSTANTS.
      WRITE (*,1) MFVNAM,VERSION
    1 FORMAT (/,34X,'MODFLOW',A,/,
     &4X,'U.S. GEOLOGICAL SURVEY MODULAR FINITE-DIFFERENCE',
     &' GROUND-WATER FLOW MODEL',/,29X,'Version ',A/)
      INUNIT = 99
C3------GET THE NAME OF THE NAME FILE
      if( len(trim(FnamePASS)).gt.1 ) then
        Fname = FnamePASS
      else
        CALL GETNAMFIL(FNAME)
      endif
      MAXUNIT= INUNIT
C
C4------OPEN NAME FILE.
       OPEN (UNIT=INUNIT,FILE=FNAME,STATUS='OLD',ACTION=ACTION(1))
      NC=INDEX(FNAME,' ')
      WRITE(*,'(A,A)')' Using NAME file: ',FNAME(1:NC)
C6------ALLOCATE AND READ (AR) PROCEDURE
      NSOL=1
      CALL GWF2BAS7AR(INUNIT,CUNIT,VERSION,24,31,32,MAXUNIT,IGRID,12,
     1                HEADNG,26,MFVNAM)
      IF(IUNIT(1).GT.0) CALL GWF2BCF7AR(IUNIT(1),IGRID)
      IF(IUNIT(23).GT.0) CALL GWF2LPF7AR(IUNIT(23),IGRID)
      IF(IUNIT(37).GT.0) CALL GWF2HUF7AR(IUNIT(37),IUNIT(47),
     1                                     IUNIT(53),IGRID)
      IF(IUNIT(21).GT.0) CALL GWF2HFB7AR(IUNIT(21),IGRID)
C7------SIMULATE EACH STRESS PERIOD.
      KKPER = 1
      KKSTP = 1
C7C1----CALCULATE TIME STEP LENGTH. SET HOLD=HNEW.
      CALL GWF2BAS7AD(KKPER,KKSTP,IGRID)
      IF(IUNIT(1).GT.0) CALL GWF2BCF7AD(KKPER,IGRID)
      IF(IUNIT(23).GT.0) CALL GWF2LPF7AD(KKPER,IGRID)
      IF(IUNIT(37).GT.0) CALL GWF2HUF7AD(KKPER,IGRID)
      KKITER = 1
C7C2A---FORMULATE THE FINITE DIFFERENCE EQUATIONS.
       CALL GWF2BAS7FM(IGRID)
       IF(IUNIT(1).GT.0) CALL GWF2BCF7FM(KKITER,KKSTP,KKPER,IGRID)
       IF(IUNIT(23).GT.0) CALL GWF2LPF7FM(KKITER,KKSTP,KKPER,IGRID)
       IF(IUNIT(37).GT.0) CALL GWF2HUF7FM(KKITER,KKSTP,KKPER,
     +   IUNIT(47),IGRID)
       IF(IUNIT(21).GT.0) CALL GWF2HFB7FM(IGRID)
       i= icbcFL
C
C--------- CUT DEALLOCATION ROUTINES and Call SEParately 
C
      return 
      END
C ______________________________________________________________________________________________
c
      subroutine MF_DumpMemory(IGRID)       
C1------USE package modules.
      USE GLOBAL
      INCLUDE 'openspec.inc'
c      
C9------CLOSE FILES AND DEALLOCATE MEMORY.  GWF2BAS7DA MUST BE CALLED
C9------LAST BECAUSE IT DEALLOCATES IUNIT.
      CALL SGWF2BAS7PNT(IGRID)
      IF(IUNIT(1).GT.0) CALL GWF2BCF7DA(IGRID)
      IF(IUNIT(21).GT.0) CALL GWF2HFB7DA(IGRID)
      IF(IUNIT(23).GT.0) CALL GWF2LPF7DA(IGRID)
      IF(IUNIT(37).GT.0) CALL GWF2HUF7DA(IGRID)
      CALL GWF2BAS7DA(IGRID)
C
      return 
      END
C ______________________________________________________________________________________________
c
      SUBROUTINE GETNAMFIL(FNAME)
C     ******************************************************************
C     GET THE NAME OF THE NAME FILE
C     ******************************************************************
C        SPECIFICATIONS:
C
C     ------------------------------------------------------------------
      CHARACTER*(*) FNAME
      CHARACTER*200 COMLIN
      LOGICAL EXISTS
C     ------------------------------------------------------------------
C
C Get name file from command line or user interaction.
        FNAME=' '
        COMLIN=' '
C *** Subroutines GETARG and GETCL are extensions to Fortran 90/95 that
C *** allow a program to retrieve command-line arguments.  To enable
C *** Modflow-2000 to read the name of a Name file from the command
C *** line, either GETARG or GETCL must be called, but not both.  As
C *** distributed, the call to GETARG is uncommented.  For compilers
C *** that support GETCL but not GETARG, comment out the call to GETARG
C *** and uncomment the call to GETCL.  The calls to both GETARG and
C *** GETCL may be commented out for compilers that do not support
C *** either extension.
        CALL GETARG(1,COMLIN)
C        CALL GETCL(COMLIN)
        ICOL = 1
        IF(COMLIN.NE.' ') THEN
          FNAME=COMLIN
        ELSE
   15     WRITE (*,*) ' Enter the name of the NAME FILE: '
          READ (*,'(A)') FNAME
          call TrimALL(fname)
          ki = len(trim(fname))
c          if(ki.lt.2) FNAME = 'Name_mf2k5_tr.txt'
c          if(ki.lt.2) FNAME = 'INPUT_WT_DIRECT.NAM.txt'
          if(ki.lt.2) FNAME = 'INPUT_PartUP.NAM.txt'
c          if(ki.lt.2) FNAME = 'INPUT_PM-SS.NAME.txt'
          CALL URWORD(FNAME,ICOL,ISTART,ISTOP,0,N,R,0,0)
          FNAME=FNAME(ISTART:ISTOP)
          IF (FNAME.EQ.' ') GOTO 15
        ENDIF
        INQUIRE (FILE=FNAME,EXIST=EXISTS)
        IF(.NOT.EXISTS) THEN
          NC=INDEX(FNAME,' ')
          FNAME(NC:NC+3)='.nam'
          INQUIRE (FILE=FNAME,EXIST=EXISTS)
          IF(.NOT.EXISTS) THEN
            WRITE (*,480) FNAME(1:NC-1),FNAME(1:NC+3)
  480       FORMAT(1X,'Can''t find name file ',A,' or ',A)
            CALL USTOP(' ')
          ENDIF
        ENDIF
C
      RETURN
      END
