C     *****************************************************************
C     MAIN CODE FOR T-COMP_Simulated
C     ******************************************************************
c Program  T-COMP_Simulated.f  -- Extracts simulated T using nodes and weights from T-COMP_Weight.f
c
c    Read regional MODFLOW files. 
c    Extracts regional gridding, T, & VCONs. 
c    Read Factors file -- RegionalNAME_TranFACTORS.txt     
c    Write output TRAN-Simulated.txt file  in the format,
c        Nsites                              !!   Number of sites to be read 
c        AquiferTestNAME     0000000.000     !!   site name (A32) & transmissivity (G16.8)       
c        AquiferTestNAME     0000000.000     !!   site name (A32) & transmissivity (G16.8)       
c        AquiferTestNAME     0000000.000     !!   site name (A32) & transmissivity (G16.8)       
c
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C1------USE package modules from MODFLOW.
      USE GLOBAL
      USE TcompReduceMODULE
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      character*5  sNOR
      character*5  sNOL
      character*5  sVAR
      CHARACTER*64 sWelNAME(nMAXwell)
      CHARACTER*64 sLITH(nMAXlith)
      CHARACTER*200 FNAME
      CHARACTER*2048 sTxT
      real rLITHprop(nMAXlith,4)
      real rWelINT(nMAXint)
      real rn(lrna)
      real rWelX(nMAXwell)
      real rWelY(nMAXwell)
      real rDDthresh
      real rRADmin 
      real rRADmax 
      INTEGER nWelINT(nMAXwell)
      INTEGER nWells
      INTEGER nZmini
      INTEGER nLITH
      INTEGER ierr
      INTEGER io
      INTEGER i
      LOGICAL(4) bYN
C1------USE package modules from T-COMP routines
C
      IGRID = 1
      ioq = 95
C
C  Get MODFLOW STUFF 
      Fname = ''
      call MF_DimNcond(IGRID,Fname) 
C ___________________  START doing my stuff ______________________________________
c
c  Convert CC&CR to a node centered T array, Convert CV to Kz, and Calculate Kz/Kh 
      call Cond2Tran(IGRID) 
c
      io = 7
c Read node, fraction file that was created by T-COMP_Extract, volume file for each site      
   5  write(*,'(A)') ' Enter name of node-fraction file, '
     +              //' created by T-COMP_Extract ....'
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 5
      if(ki.lt.1) sTxT = 'OUTPUT_Tcomp_Node-Fractions.txt'
      open(io, file=trim(sTxT),ACTION='READ',err=99)
c
c  Open simulated transmissivity file       
  15  write(*,'(A)') ' Enter name of simulated T output file....'
      read(*,'(A)') sTxT
      call TrimALL(sTxT)
      ki = len(trim(sTxT))
c      if(ki.lt.1) goto 15
      if(ki.lt.1) sTxT = 'ModelNAME_T-Simulated.txt'
      open(io+1, file=trim(sTxT),ACTION='WRITE')
c  Sum T*fraction for all nodes intersected by an aquifer test
      call SumTfrac(iGRID, io, io+1) 
c
      close(io+1)
      close(io)
  99  continue    
c
C ___________________  Stop doing my stuff ______________________________________
C
C9------CLOSE FILES AND DEALLOCATE MEMORY. 
      call MF_DumpMemory(IGRID)
c---------------------------      
      CALL USTOP(' ')
      END
C ______________________________________________________________________________________________
c
      subroutine SumTfrac(iGRID, ioIN, ioOUT)
      USE GLOBAL
      USE GWFBASMODULE,ONLY:PERTIM,TOTIM,IHEDUN
      USE TcompReduceMODULE
      include '..\T-COMP_SUBS-Common\T-COMP_Mini.inc'
      CHARACTER*32 sWelNAME
      CHARACTER*2048 sTxT
      real rTRAN, rTadd
      real rFrac
      INTEGER ioIN, ioOUT
      INTEGER i,j,k,nMF  n,nNODES
      INTEGER ki
      LOGICAL(4) bYN
c
C  Get MODFLOW STUFF 
      call SGWF2BAS7PNT(IGRID)
c
c  Scan node-Fraction file      
    5 read(ioIN,'(a)',end=99) sTxT
        sWelNAME = trim(sTxT)
        if( index(sWelNAME,' ').le.1 )  goto 99   
        read(ioIN,*,err=99) nNODES
c   Convert to areas, ADD total area per regional layer, and count nodes of interest  
        rTRAN = 0.000000000000000000
        do n = 1,nNODES
          read(ioIN,*) nMF, rFRAC
          k = int((nMF-1)/NCOL/NROW) + 1
          nMF = nMF-(k-1)*NCOL*NROW
          j = int((nMF-1)/NCOL) + 1
          i = nMF-(j-1)*NCOL
          rTadd = rTbar(i,j,k) * rFrac 
          rTRAN = rTRAN + rTadd
        enddo   
c   Scan and write nodes & fractions 
        write(ioOUT,'(A32,2G16.8)') sWelNAME, rTRAN, log10(rTRAN)
      goto 5
   99 continue
c
      return 
      end     
